# Create a class that extends MaterialsObserver
class MatObserver < Sketchup::MaterialsObserver

   def onMaterialAdd(mats, mat)
      puts "Added " + mat.name + " to the model."
   end

   def onMaterialSetCurrent(mats, mat) 
      puts "Setting " + mat.name + " to current."
   end

   def onMaterialChange(mats, mat) 
      puts "Changed " + mat.name + "."
   end

   def onMaterialRefChange(mats, mat) 
      puts "The material " + mat.name + " has been applied."
   end
   
end

# Associate the observer with the Materials object
mats = Sketchup.active_model.materials
obs = MatObserver.new
mats.add_observer obs

# Create a cube at the origin
ents = Sketchup.active_model.entities
face = ents.add_face [0,0,0], [3,0,0], [3,3,0], [0,3,0]
face.reverse!
face.pushpull 3
Sketchup.active_model.active_view.zoom_extents

# Add two materials to the model
puts "Adding materials... "
mats = Sketchup.active_model.materials
mat1 = mats.add "Diamond"
mat2 = mats.add "Brick"
mat1.texture =  Sketchup.find_support_file "Ch9/diamond.jpg", "Plugins"
mat2.texture =  Sketchup.find_support_file "Ch9/brick.jpg", "Plugins"

# Apply the materials to the top and bottom faces
puts "Applying materials to entities... "
face1 = ents.find {|e| (e.typename == "Face") && 
   (e.normal == [0, 0, 1])}
face1.material = mat1
face2 = ents.find {|e| (e.typename == "Face") && 
   (e.normal == [0, -1, 0])}
face2.material = mat2

# Set the model's current material
puts "Setting the current material..."
mats.current = mat1

# Change the materials
puts "Changing texture and color..."
mat1.texture =  Sketchup.find_support_file "Ch9/brick.jpg", "Plugins"
mat2.texture =  Sketchup.find_support_file "Ch9/diamond.jpg", "Plugins"
mat1.color = [255, 0, 0]
mat2.color = [0, 0, 255]

# Remove the observer
mats.remove_observer obs